fl


\ Initialize APDS-9660
\ ( -- )
: init_APDS-9660
\ Set default value to registers
ATIME APDS9960_ATIME APDS_i2c_wr                  \ ATIME[h81]
WTIME APDS9960_WTIME APDS_i2c_wr                  \ WTIME[h83]
AILT APDS9960_AILTL setALS_Int_Thresh             \ AILT[h84:h85]
AIHT APDS9960_AIHTL setALS_Int_Thresh             \ AIHT[h86:h87]
PILT APDS9960_PILT APDS_i2c_wr                    \ PILT[h89]
PIHT APDS9960_PIHT APDS_i2c_wr                    \ PIHT[h8B]
PERS APDS9960_PERS APDS_i2c_wr                    \ PERS[h8C]
CONFIG1 APDS9960_CONFIG1 APDS_i2c_wr              \ CONFIG1[h8D]
PROX_PPULSE APDS9960_PPULSE APDS_i2c_wr           \ PPUL[h8E]
LDRIVE setLED                                     \ CONTROL[h8F 7:6]
PGAIN setPGAIN                                    \ CONTROL[h8F 3:2]
AGAIN setAGAIN                                    \ CONTROL[h8F 1:0]
\ CONFIG2 APDS9960_CONFIG2 APDS_i2c_wr              \ CONFIG2[h90]
POFFSET_UR APDS9960_POFFSET_UR APDS_i2c_wr        \ POFFSET_UR[h9D]
POFFSET_DL APDS9960_POFFSET_DL APDS_i2c_wr        \ POFFSET_DL[h9E]
CONFIG3 APDS9960_CONFIG3 APDS_i2c_wr              \ CONFIG3[9F]
GPENTH APDS9960_GPENTH APDS_i2c_wr                \ GPENTH[hA0]
GEXTH APDS9960_GEXTH APDS_i2c_wr                  \ GEXTH[hA1]
GCONF1 APDS9960_GCONF1 APDS_i2c_wr                \ GCONF1[hA2]
GGAIN setGGAIN                                    \ GCONF2[hA3 6:5]
GLDRIVE setGLDRIVE                                \ GCONF2[hA3 4:3]
GWTIME setGWTIME                                  \ GCONF2[hA3 2:0]
GOFFSET APDS9960_GOFFSET_U APDS_i2c_wr            \ GOFFSET_U[hA4]
GOFFSET APDS9960_GOFFSET_D APDS_i2c_wr            \ GOFFSET_D[hA5]
GPULSE APDS9960_GPULSE APDS_i2c_wr                \ GPULSE[hA6]
GOFFSET APDS9960_GOFFSET_L APDS_i2c_wr            \ GOFFSET_L[hA7]
GOFFSET APDS9960_GOFFSET_R APDS_i2c_wr            \ GOFFSET_R[hA9]
GCONF3 APDS9960_GCONF3 APDS_i2c_wr                \ GCONF3[hAA]
GIEN setGIEN                                      \ GCONF4[hAB]
;

\ Check if register has no value
\ ( n1 -- n2 ) n1:register  n2:true(available)
: chkReg
h82 over =          \ case h82
if 0
else h8A over =     \ case h8A
if 0
else h91 over =     \ case h91
if 0
else hA8 over =     \ case hA8
if 0
else hAC over =     \ case hAC
if 0
else hAD over =     \ case hAD
if 0
else 1
thens
nip 
;

\ Display registers
\ ( -- )
: dispReg
hex
." Register[h]" tab ." Value[h]" cr
h80                                \ ( register )
d48 0 do                           
     dup                           \ ( register register )
     chkReg
     \ Valid register
     if
          dup dup                  \ ( register register register )
          .byte 2tab    
          APDS_i2c_rd . cr         \ ( register )
     then
     1+                            \ ( register+1 )
loop
drop
decimal 
;

\ Enable power
\ ( -- )  
: enablePower APDS9960_PON 1 APDS9960_ENABLE bitControl ;

\ Disable power
\ ( -- )  
: disablePower APDS9960_PON 0 APDS9960_ENABLE bitControl ;


\ Test color/ALS
\ ( -- )
: color_test
\ Initialize
6 delms
init_APDS-9660
\ Power on
enablePower
\ Enable AEM to enter into color/ALS engine
APDS9960_AEN 1 APDS9960_ENABLE bitControl
7 delms
begin
     APDS9960_CDATAL rd16bit .     \ Get clear channel data[16bit]
     APDS9960_RDATAL rd16bit .     \ Get red channel data[16bit]
     APDS9960_GDATAL rd16bit .     \ Get green channel data[16bit]
     APDS9960_BDATAL rd16bit .     \ Get blue channel data[16bit]
     cr
fkey? swap drop
1000 delms
until
;

\ Test proximity
\ ( -- )
: proximity_test
\ Initialize
6 delms
init_APDS-9660
\ Power on
enablePower
\ Enable PEN to enter into proximity engine
APDS9960_PEN 1 APDS9960_ENABLE bitControl
PGAIN_2X setPGAIN
7 delms
begin
     APDS9960_PDATA APDS_i2c_rd .
     cr
fkey? swap drop
d250 delms
until
;

\ Watch INTpin
\ ( -- )
: watchINT
begin 
     ina COG@ 1 INTpin lshift and
     if 1 interrupt W! then
     begin interrupt W@ 0= until
until
;

\ Process Gesture Data
\ ( -- t/f )  t:success       fl
: procGesture
\ Clear parameter
0 u_first L! 0 u_first 4+ L!
0 u_last L! 0 u_last 4+ L!
\ Search *_first
gestureDataTotal W@ 0 do
     FIFO i 4* +                        \ ( FIFOaddr )
     4 0 do
          dup                           \ ( FIFOaddr FIFOaddr )
          C@ Gesture_Threshold >        \ ( FIFOaddr 1/0 )
          swap 1+                       \ ( 1/0 FIFOaddr+1 ) 
     loop
                                        \ ( 1/0 1/0 1/0 1/0 FIFOaddr )
     drop and and and                   \ ( 1/0 ) 
     if
          \ Save data to u_first/d_first/l_first/r_first
          u_first FIFO i 4* +           \ ( u_first FIFOaddr ) 
          4 0 do 
               2dup                     \ ( u_first FIFOaddr u_first FIFOaddr )
               C@ swap W!
               swap 2+ swap 1+          \ ( u_first+2 FIFOaddr+1 )
          loop         
          2drop                         \ ( -- ) 
          gestureDataTotal W@ seti      \ Break loop
          1 procErr W!
     else
          0 procErr W!       
     then
loop
\ --- Debug ---------------------------------------------------------------------------------------
[ifdef debug
prtUDLR
." Gestute FIFO Data Total:" gestureDataTotal W@ . cr
." u_first dump" cr
u_first 8 dump
." process error:" procErr W@ 0= if ." error" else ." no" then cr 
]
\ -------------------------------------------------------------------------------------------------
procErr W@ 
if
     \ Search *_last
     gestureDataTotal W@ 0 do
          FIFO 
          gestureDataTotal W@ 1- i - 4* +              \ ( FIFOaddr )
          4 0 do
               dup                                     \ ( FIFOaddr FIFOaddr )
               C@ Gesture_Threshold >                  \ ( FIFOaddr 1/0 )
               swap 1+                                 \ ( 1/0 FIFOaddr+1 ) 
          loop                               
                                                       \ ( 1/0 1/0 1/0 1/0 FIFOaddr )
          drop and and and                             \ ( 1/0 )                                   
          if
               \ Save data to u_last/d_last/l_last/r_last
               u_last 
               FIFO gestureDataTotal W@ 1- i - 4* +    \ ( u_last FIFOaddr ) 
               4 0 do 
                    2dup                               \ ( u_last FIFOaddr u_last FIFOaddr )
                    C@ swap W!
                    swap 2+ swap 1+                    \ ( u_last+2 FIFOaddr+1 ) 
               loop
               2drop                                   \ ( -- )
               gestureDataTotal W@ seti                \ Break loop
               1 procErr W!
          else
               0 procErr W!       
          then
     loop                                                            \      ." ------B-------" st?
\ --- Debug ---------------------------------------------------------------------------------------
[ifdef debug 
." u_last dump" cr
u_last 8 dump
." process error:" procErr W@ 0= if ." error" else ." no" then cr 
]
\ -------------------------------------------------------------------------------------------------
     procErr W@
     if
          \ Calculate first vs last of up/down and left/right
          u_first W@ u_first 2+ W@ - d100 * u_first W@ u_first 2+ W@ + / ud_ratio_first L!
          u_first 4 + W@ u_first 6 + W@ - d100 * u_first 4 + W@ u_first 6 + W@ + / lr_ratio_first L!          
          u_last W@ u_last 2+ W@ - d100 * u_last W@ u_last 2+ W@ + / ud_ratio_last L!
          u_last 4 + W@ u_last 6 + W@ - d100 * u_last 4 + W@ u_last 6 + W@ + / lr_ratio_last L!
\ --- Debug ---------------------------------------------------------------------------------------
[ifdef debug
." ud_ratio_first lr_ratio_first ud_ratio_last lr_ratio_last" cr
ud_ratio_first L@ . lr_ratio_first L@ . ud_ratio_last L@ . lr_ratio_last L@ . cr
]
\ -------------------------------------------------------------------------------------------------
          \ Determine the difference between the first and last ratio     
          ud_ratio_last L@ ud_ratio_first L@ - ud_delta L!
          lr_ratio_last L@ lr_ratio_first L@ - lr_delta L!
          
\ --- Debug ---------------------------------------------------------------------------------------
[ifdef debug 
." ud_delta lr_delta" cr
ud_delta L@ . lr_delta L@ . cr
]
\ -------------------------------------------------------------------------------------------------
          \ Determine U/D gesture
          ud_delta L@ Gesture_Sensivity1 >=
          if
               1 ud_count L!     
          else
               ud_delta L@ Gesture_Sensivity1 invert 1+ <=
               if
                    -1 ud_count L!
               else
                    0 ud_count L!
          thens
          \ Determine L/R gesture
          lr_delta L@ Gesture_Sensivity1 >=
          if
               1 lr_count L!
          else
               lr_delta L@ Gesture_Sensivity1 invert 1+ <=
               if
                    -1 lr_count L!
               else
                    0 lr_count L!
          thens         
          \ Determine Near/Far gesture
          ud_count L@ 0= lr_count L@ 0= and
          if
               ud_delta L@ abs Gesture_Sensivity2 < lr_delta L@ abs Gesture_Sensivity2 < and
               if
                    ud_delta L@ 0= lr_delta L@ 0= and
                    if
                         \ near state
                    else
                         ud_delta L@ 0 <> lr_delta L@ 0 <> and
                         if
                             \ far state
                         then
                    then
               then
          else
               ud_delta L@ abs Gesture_Sensivity2 < lr_delta L@ abs Gesture_Sensivity2 < and
               if
                    ud_delta L@ 0= lr_delta L@ 0= and
                    if
                         \ 1 near_count L!
                    then
                    if
                         
                    then
               then
          then
                                            
          
\ --- Debug ---------------------------------------------------------------------------------------
[ifdef debug 
." ud_count lr_count" cr    
ud_count L@ . lr_count L@ . cr
]
\ -------------------------------------------------------------------------------------------------
thens
;

\ Determine swipe direction
\ ( -- n1 ) n1:direction code
: decdodeGesture
1 decordErr W!  
ud_count L@ -1 = lr_count L@ 0= and
if 
     1              \ UP
else 
     ud_count L@ 1 = lr_count L@ 0= and
     if 
          2         \ DOWN
     else
          ud_count L@ 0= lr_count L@ 1 = and
          if
               3    \ RIGHT
          else
               ud_count L@ 0= lr_count L@ -1 = and
               if
                    4    \ LEFT
               else
                    ud_count L@ -1 = lr_count L@ 1 = and
                    if
                         ud_delta L@ abs lr_delta L@ abs >
                         if
                              1    \ UP
                         else
                              3    \ RIGHT
                         then
                    else
                         ud_count L@ 1 = lr_count L@ -1 = and
                         if
                              ud_delta L@ abs lr_delta L@ abs >
                              if
                                   2    \ DOWN
                              else
                                   4    \ LEFT
                              then
                         else
                              ud_count L@ -1 = lr_count L@ -1 = and
                              if                        
                                   ud_delta L@ abs lr_delta L@ abs >
                                   if
                                        1    \ UP
                                   else
                                        4    \ LEFT
                                   then
                              else
                                   ud_count L@ 1 = lr_count L@ 1 = and
                                   if
                                        ud_delta L@ abs lr_delta L@ abs >
                                        if
                                             2    \ DOWN
                                        else
                                             3    \ RIGHT
                                        then
                                   else
                                        0 decordErr W!
                                   then
thens
;

